@@ -10,8 +10,12 @@ def loop(self, n=None):
     """
     Returns a clip that plays the current clip in an infinite loop.
     Ideal for clips coming from gifs.
-    :param n: number of times the clip should be played. If `None` the
-        the clip will loop indefinitely (i.e. with no set duration).
+    
+    Parameters
+    ------------
+    n
+      Number of times the clip should be played. If `None` the
+      the clip will loop indefinitely (i.e. with no set duration).
     """
     result = self.fl_time(lambda t: t % self.duration)
     if n:
